@@ -1,17 +1,23 @@
-coeftest <- function(x, vcov = NULL, df = NULL, ...)
+coeftest <- function(x, vcov. = NULL, df = NULL, ...)
 {
   UseMethod("coeftest")
 }
 
-coeftest.default <- function(x, vcov = NULL, df = NULL, ...)
+coeftest.default <- function(x, vcov. = NULL, df = NULL, ...)
 {
-  est <- as.vector(coef(x))
-  if(is.null(vcov)) se <- vcov(x) else {
-      if(is.function(vcov)) se <- vcov(x)
-        else se <- vcov
+  est <- coef(x)
+  if(is.null(vcov.)) se <- vcov(x) else {
+      if(is.function(vcov.)) se <- vcov.(x)
+        else se <- vcov.
   }
   se <- sqrt(diag(se))
-  tval <- est/se
+
+  if(!is.null(names(est)) && !is.null(names(se))) {
+    anames <- names(est)[names(est) %in% names(se)]
+    est <- est[anames]
+    se <- se[anames]
+  }  
+  tval <- as.vector(est)/se
 
   if(is.null(df)) df <- x$df.residual
   if(is.null(df)) df <- 0
@@ -33,20 +39,44 @@ coeftest.default <- function(x, vcov = NULL, df = NULL, ...)
   return(rval)
 } 
 
-coeftest.breakpointsfull <- function(x, vcov = NULL, df = NULL, ...)
+coeftest.survreg <- function(x, vcov. = NULL, df = Inf, ...)
+{
+  if(is.finite(df) && df > 0) mthd <- "t"
+    else mthd <- "z"
+  mthd <- paste(mthd, "test of coefficients of",
+    dQuote(class(x)[1]), "object", sQuote(deparse(substitute(x))))
+  if(is.null(vcov.)) v <- vcov(x) else {
+      if(is.function(vcov.)) v <- vcov.(x)
+        else v <- vcov.
+  }
+  if(length(x$coefficients) < NROW(x$var)) {
+    x$coefficients <- c(x$coefficients, "Log(scale)" = log(x$scale))
+  }
+  rval <- coeftest.default(x, vcov. = v, df = df, ...)  
+  attr(rval, "method") <- mthd
+  return(rval)
+} 
+
+coeftest.breakpointsfull <- function(x, vcov. = NULL, df = NULL, ...)
 {
   stopifnot(require("strucchange"))
   
   est <- coef(x, ...)
+
+  if(is.null(df)) {
+    df <- df.residual(x, ...)
+    df <- as.vector(rep(df, rep(NCOL(est), length(df))))
+  }  
+
   rnames <- as.vector(t(outer(rownames(est), colnames(est), paste)))
   est <- as.vector(t(est))
   
-  se <- vcov.breakpointsfull(x, vcov = vcov, ...)
+  se <- vcov.breakpointsfull(x, vcov. = vcov., ...)
 
   se <- as.vector(sapply(seq(along = se), function(x) sqrt(diag(se[[x]]))))
   tval <- est/se
 
-  if(is.finite(df) && df > 0) {
+  if(any(is.finite(df) && df > 0)) {
     pval <- 2 * pt(abs(tval), df = df, lower.tail = FALSE)
     cnames <- c("Estimate", "Std. Error", "t value", "Pr(>|t|)")
     mthd <- "t"
